from django.db import models
from django.test import TestCase
from altimate_django.management.commands.checks.reverse_cascade import ReverseCascade
from altimate_django.management.commands.models.field_info import FieldInfo


class ReverseCascadeRelatedModel(models.Model):
    pass


class ReverseCascadeTestModelA(models.Model):
    related_model = models.ForeignKey(
        ReverseCascadeRelatedModel, on_delete=models.CASCADE
    )


class ReverseCascadeTestModelB(models.Model):
    related_model = models.OneToOneField(
        ReverseCascadeRelatedModel, on_delete=models.CASCADE
    )


class ReverseCascadeTestModelC(models.Model):
    related_model = models.ForeignKey(
        ReverseCascadeRelatedModel, on_delete=models.CASCADE, null=True
    )


class ReverseCascadeTestModelD(models.Model):
    related_model = models.ForeignKey(
        ReverseCascadeRelatedModel, on_delete=models.PROTECT
    )


class TestReverseCascade(TestCase):
    def test_reverse_cascade_detected(self):
        fields_to_test = [
            (
                ReverseCascadeTestModelA._meta.get_field("related_model"),
                ReverseCascadeTestModelA,
                True,
            ),
            (
                ReverseCascadeTestModelB._meta.get_field("related_model"),
                ReverseCascadeTestModelB,
                True,
            ),
            (
                ReverseCascadeTestModelC._meta.get_field("related_model"),
                ReverseCascadeTestModelC,
                False,
            ),
            (
                ReverseCascadeTestModelD._meta.get_field("related_model"),
                ReverseCascadeTestModelD,
                False,
            ),
        ]

        for field, model, should_detect_issue in fields_to_test:
            field_info = FieldInfo(name=field.name, model=model.__name__, field=field)
            check = ReverseCascade(field_info)
            result = check.perform_field_check()

            if should_detect_issue:
                self.assertIsNotNone(
                    result,
                    f"ReverseCascade check should detect an issue in {field.name}",
                )
            else:
                self.assertIsNone(
                    result,
                    f"ReverseCascade check should not detect an issue in {field.name}",
                )



from django.db import models
from django.test import TestCase
from altimate_django.management.commands.checks.inappropriate_cascade import (
    InappropriateCascade,
)
from altimate_django.management.commands.models.field_info import FieldInfo


class InappropriateCascadeTestModelC(models.Model):
    pass


class InappropriateCascadeTestModelD(models.Model):
    cascade_field = models.ForeignKey(
        InappropriateCascadeTestModelC,
        on_delete=models.CASCADE,
        related_name="cascade_related",
    )
    set_null_field = models.ForeignKey(
        InappropriateCascadeTestModelC,
        on_delete=models.SET_NULL,
        null=True,
        related_name="set_null_related",
    )


class InappropriateCascadeTestCase(TestCase):
    def test_inappropriate_cascade(self):
        # Test a ForeignKey with CASCADE on_delete
        field_info_cascade = FieldInfo(
            name="cascade_field",
            model=InappropriateCascadeTestModelD,
            field=InappropriateCascadeTestModelD._meta.get_field("cascade_field"),
        )
        check_cascade = InappropriateCascade(field_info_cascade)
        result_cascade = check_cascade.perform_field_check()
        self.assertIsNotNone(
            result_cascade,
            "InappropriateCascade check should detect an issue in cascade_field",
        )
        self.assertEqual(result_cascade["severity"], "Medium")
        self.assertIn(
            "Inappropriate use of CASCADE on ForeignKey", result_cascade["description"]
        )

        # Test a ForeignKey with SET_NULL on_delete
        field_info_set_null = FieldInfo(
            name="set_null_field",
            model=InappropriateCascadeTestModelD,
            field=InappropriateCascadeTestModelD._meta.get_field("set_null_field"),
        )
        check_set_null = InappropriateCascade(field_info_set_null)
        self.assertIsNone(
            check_set_null.perform_field_check(),
            "InappropriateCascade check should not detect an issue in set_null_field",
        )



import pprint
from django.core.management.base import BaseCommand
from django.apps import apps
from tabulate import tabulate
from termcolor import colored
from .models.model_info import ModeInfo
from .models.field_info import FieldInfo
from .checks.reverse_cascade import ReverseCascade
from .checks.minmax_validator import MinMaxValidator
from .checks.excessive_nulls import ExcessiveNulls
from .checks.cascade_for_fk_onetoone import CascadeForFKOneToOne
from .checks.inappropriate_cascade import InappropriateCascade
from .checks.nullable_unique_fields import NullableUniqueFields
from .checks.unbounded_autoincrement_pk import UnboundedAutoIncrementPK
from .checks.cascade_on_foreign_key import CascadeOnForeignKey
from .checks.reserved_sql_keywords import ReservedSQLKeywords
from .checks.foreign_key_naming import ForeignKeyNaming
from .checks.excessive_blanks import ExcessiveBlanks
from .checks.missing_help_text import MissingHelpText
from .checks.large_charfield import LargeCharField
from .checks.unique_without_index import UniqueWithoutIndex
from .checks.missing_default import MissingDefault


FIELD_CHECK_CLASSES = [
    ReverseCascade,
    MinMaxValidator,
    ExcessiveNulls,
    CascadeForFKOneToOne,
    InappropriateCascade,
    NullableUniqueFields,
    UnboundedAutoIncrementPK,
    CascadeOnForeignKey,
    ReservedSQLKeywords,
    ForeignKeyNaming,
    ExcessiveBlanks,
    MissingHelpText,
    LargeCharField,
    UniqueWithoutIndex,
    MissingDefault,
]


class Command(BaseCommand):
    help = "Prints the schema of all models and their entity relationship"

    severity_colors = {
        "High": "red",
        "Warning": "yellow",
        "Low": "cyan",
    }

    def collect_check_data(self, model):
        return ModeInfo(
            model_name=model.__name__,
            object_count=model.objects.count(),
            has__str__=hasattr(model, "__str__"),
            has__unicode__=hasattr(model, "__unicode__"),
            fields=[
                self.create_field_schema(model, field)
                for field in model._meta.get_fields()
            ],
        )

    def create_field_schema(self, model, field):
        field_schema_args = {
            "name": field.name,
            "model": model.__name__,
            "field": field,
        }

        return FieldInfo(**field_schema_args)

    def fetch_schema(self):
        model_schemas = [
            ModeInfo(
                model_name=model.__name__,
                fields=[
                    self.create_field_schema(model, field)
                    for field in model._meta.get_fields()
                ],
            )
            for model in apps.get_models()
        ]

        schema_list = [model_schema.to_dict() for model_schema in model_schemas]
        pp = pprint.PrettyPrinter(width=38, compact=True)
        pp.pprint(schema_list)

    def check_data_issues(self, model_info: ModeInfo):
        recommendations = []
        for field_info in model_info.fields:
            for check in FIELD_CHECK_CLASSES:
                result = check(field_info).perform_field_check()
                if result:
                    recommendations.append(
                        (model_info.model_name, field_info.name, result)
                    )
        return recommendations

    def create_table_data(self, recommendations):

        table_data = []
        for model_name, field_name, recommendation in recommendations:
            if field_name and recommendation:
                severity, description, explanation = (
                    recommendation["severity"],
                    recommendation["description"],
                    recommendation["explanation"],
                )
                color = self.severity_colors.get(severity, "green")

                table_data.append(
                    [
                        f"{model_name}.{field_name}",
                        colored(severity, color),
                        description,
                        explanation,
                    ]
                )
        return table_data

    def handle(self, *args, **options):
        recommendations = []

        for model in apps.get_models():
            model_info = self.collect_check_data(model)
            recommendations.extend(self.check_data_issues(model_info))

        if recommendations:
            self.stdout.write("Data Issues Detected:")

            headers = ["Model.Field", "Severity", "Description", "Explanation"]
            table_data = self.create_table_data(recommendations)

            table_str = tabulate(
                table_data,
                headers=headers,
                tablefmt="grid",
                maxcolwidths=[None, None, 60, 60],
            )
            self.stdout.write(table_str)

        else:
            self.stdout.write("No data issues detected.")



from django.test import TestCase
from django.db import models
from django.db import models
from altimate_django.management.commands.checks.unique_without_index import (
    UniqueWithoutIndex,
)
from altimate_django.management.commands.models.field_info import FieldInfo


class UniqueWithoutIndexTestModelA(models.Model):
    field1 = models.CharField(max_length=100, unique=True, db_index=True)


class UniqueWithoutIndexTestModelB(models.Model):
    field2 = models.CharField(max_length=100, unique=True, db_index=False)


class UniqueWithoutIndexTestModelC(models.Model):
    field3 = models.CharField(max_length=100, unique=False, db_index=True)


class UniqueWithoutIndexTestModelD(models.Model):
    field4 = models.CharField(max_length=100, unique=False, db_index=False)


class TestUniqueWithoutIndex(TestCase):
    def test_unique_without_index_detected(self):
        fields_to_test = [
            (
                UniqueWithoutIndexTestModelA._meta.get_field("field1"),
                UniqueWithoutIndexTestModelA,
                False,
            ),
            (
                UniqueWithoutIndexTestModelB._meta.get_field("field2"),
                UniqueWithoutIndexTestModelB,
                True,
            ),
            (
                UniqueWithoutIndexTestModelC._meta.get_field("field3"),
                UniqueWithoutIndexTestModelC,
                False,
            ),
            (
                UniqueWithoutIndexTestModelD._meta.get_field("field4"),
                UniqueWithoutIndexTestModelD,
                False,
            ),
        ]

        for field, model, should_detect_issue in fields_to_test:
            field_info = FieldInfo(name=field.name, model=model.__name__, field=field)
            check = UniqueWithoutIndex(field_info)
            result = check.perform_field_check()

            if should_detect_issue:
                self.assertIsNotNone(
                    result,
                    f"UniqueWithoutIndex check should detect an issue in {field.name}",
                )
            else:
                self.assertIsNone(
                    result,
                    f"UniqueWithoutIndex check should not detect an issue in {field.name}",
                )



from django.test import TestCase
from django.db import models
from django.db import models
from altimate_django.management.commands.checks.cascade_on_foreign_key import (
    CascadeOnForeignKey,
)
from altimate_django.management.commands.models.field_info import FieldInfo


class CascadeOnForeignKeyTestModelA(models.Model):
    pass


class CascadeOnForeignKeyTestModelB(models.Model):
    cascade_field = models.ForeignKey(
        CascadeOnForeignKeyTestModelA,
        on_delete=models.CASCADE,
        related_name="cascade_related",
    )
    set_null_field = models.ForeignKey(
        CascadeOnForeignKeyTestModelA,
        on_delete=models.SET_NULL,
        null=True,
        related_name="set_null_related",
    )
    protect_field = models.ForeignKey(
        CascadeOnForeignKeyTestModelA,
        on_delete=models.PROTECT,
        related_name="protect_related",
    )


class TestCascadeOnForeignKey(TestCase):
    def test_cascade_on_foreign_key(self):
        field_info = FieldInfo(
            field=CascadeOnForeignKeyTestModelB._meta.get_field("cascade_field"),
            name="cascade_field",
            model=CascadeOnForeignKeyTestModelB,
        )
        check = CascadeOnForeignKey(field_info)
        result = check.perform_field_check()
        self.assertIsNotNone(result)

    def test_set_null_on_foreign_key(self):
        field_info = FieldInfo(
            field=CascadeOnForeignKeyTestModelB._meta.get_field("set_null_field"),
            name="set_null_field",
            model=CascadeOnForeignKeyTestModelB,
        )
        check = CascadeOnForeignKey(field_info)
        result = check.perform_field_check()
        self.assertIsNone(result)

    def test_protect_on_foreign_key(self):
        field_info = FieldInfo(
            field=CascadeOnForeignKeyTestModelB._meta.get_field("protect_field"),
            name="protect_field",
            model=CascadeOnForeignKeyTestModelB,
        )
        check = CascadeOnForeignKey(field_info)
        result = check.perform_field_check()
        self.assertIsNone(result)



from django.db import models
from django.test import TestCase
from altimate_django.management.commands.checks.large_charfield import LargeCharField
from altimate_django.management.commands.models.field_info import FieldInfo


class LargeCharFieldTestModelA(models.Model):
    normal_charfield = models.CharField(max_length=100)


class LargeCharFieldTestModelB(models.Model):
    large_charfield = models.CharField(max_length=500)


class LargeCharFieldTestCase(TestCase):
    def test_large_char_field(self):
        # Test a CharField with a max_length below the threshold
        field_info_normal_charfield = FieldInfo(
            name="normal_charfield",
            model=LargeCharFieldTestModelA,
            field=LargeCharFieldTestModelA._meta.get_field("normal_charfield"),
        )
        check_normal_charfield = LargeCharField(field_info_normal_charfield)
        self.assertIsNone(
            check_normal_charfield.perform_field_check(),
            "LargeCharField check should not detect an issue in normal_charfield",
        )

        # Test a CharField with a max_length above the threshold
        field_info_large_charfield = FieldInfo(
            name="large_charfield",
            model=LargeCharFieldTestModelB,
            field=LargeCharFieldTestModelB._meta.get_field("large_charfield"),
        )
        check_large_charfield = LargeCharField(field_info_large_charfield)
        result = check_large_charfield.perform_field_check()
        self.assertIsNotNone(
            result, "LargeCharField check should detect an issue in large_charfield"
        )
        self.assertEqual(result["severity"], "Warning")
        self.assertIn(
            "Consider using TextField instead of CharField", result["description"]
        )



from django.db import models
from django.test import TestCase
from altimate_django.management.commands.checks.missing_help_text import MissingHelpText
from altimate_django.management.commands.models.field_info import FieldInfo


class MissingHelpTextTestModelA(models.Model):
    name = models.CharField(max_length=100)
    description = models.CharField(
        max_length=100, help_text="A short description of the model."
    )


class MissingHelpTextTestModelB(models.Model):
    name = models.CharField(max_length=100)
    description = models.CharField(max_length=100)


class MissingHelpTextTestCase(TestCase):
    def test_missing_help_text(self):
        # Test a field with help_text
        field_info_with_help_text = FieldInfo(
            name="description",
            model=MissingHelpTextTestModelA,
            field=MissingHelpTextTestModelA._meta.get_field("description"),
        )
        check_with_help_text = MissingHelpText(field_info_with_help_text)
        self.assertIsNone(
            check_with_help_text.perform_field_check(),
            "MissingHelpText check should not detect an issue in description field with help_text",
        )

        # Test a field without help_text
        field_info_without_help_text = FieldInfo(
            name="description",
            model=MissingHelpTextTestModelB,
            field=MissingHelpTextTestModelB._meta.get_field("description"),
        )
        check_without_help_text = MissingHelpText(field_info_without_help_text)
        result = check_without_help_text.perform_field_check()
        self.assertIsNotNone(
            result,
            "MissingHelpText check should detect an issue in description field without help_text",
        )
        self.assertEqual(result["severity"], "Low")
        self.assertIn("does not have help_text", result["description"])



# setup.py
from setuptools import setup, find_packages

setup(
    name="altimate-django",
    version="0.1.3",
    packages=find_packages(),
    install_requires=["Django>=2.0", "tabulate>=0.9.0", "termcolor>=1.1.0"],
    author="Altimate Inc.",
    author_email="info@altimate.ai",
    exclude_package_data={"": ["my_tennis_club"]},
    description="A Django extension to check for various data issues",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



from django.test import TestCase
from django.db import models
from django.db import models
from altimate_django.management.commands.checks.excessive_blanks import ExcessiveBlanks
from altimate_django.management.commands.models.field_info import FieldInfo


class ExcessiveBlanksTestModelA(models.Model):
    no_blank = models.CharField(max_length=255)
    excessive_blank = models.CharField(max_length=255, blank=True)


class ExcessiveBlanksTestExcessiveBlanks(TestCase):
    def test_no_blank(self):
        field_info = FieldInfo(
            field=ExcessiveBlanksTestModelA._meta.get_field("no_blank"),
            name="no_blank",
            model=ExcessiveBlanksTestModelA,
        )
        check = ExcessiveBlanks(field_info)
        result = check.perform_field_check()
        self.assertIsNone(result)

    def test_excessive_blank(self):
        field_info = FieldInfo(
            field=ExcessiveBlanksTestModelA._meta.get_field("excessive_blank"),
            name="excessive_blank",
            model=ExcessiveBlanksTestModelA,
        )
        check = ExcessiveBlanks(field_info)
        result = check.perform_field_check()
        expected_result = {
            "field_name": "excessive_blank",
            "severity": "Info",
            "description": "Excessive use of blank=True",
            "explanation": "Using blank=True excessively may cause issues with data consistency and validation. Review your use of blank=True in this field and consider removing it if it's not necessary.",
        }
        self.assertEqual(result, expected_result)



from django.db import models
from django.test import TestCase
from altimate_django.management.commands.checks.foreign_key_naming import (
    ForeignKeyNaming,
)
from altimate_django.management.commands.models.field_info import FieldInfo


class RelatedModel(models.Model):
    name = models.CharField(max_length=255)


class ForeignKeyNamingTestModelA(models.Model):
    relatedmodel_id = models.ForeignKey(RelatedModel, on_delete=models.CASCADE)


class ForeignKeyNamingTestModelB(models.Model):
    incorrect_related_name = models.ForeignKey(RelatedModel, on_delete=models.CASCADE)


class TestForeignKeyNaming(TestCase):
    def test_correct_naming_convention(self):
        field_info = FieldInfo(
            field=ForeignKeyNamingTestModelA._meta.get_field("relatedmodel_id"),
            name="relatedmodel_id",
            model=ForeignKeyNamingTestModelA,
        )
        check = ForeignKeyNaming(field_info)
        result = check.perform_field_check()
        self.assertIsNone(result)

    def test_incorrect_naming_convention(self):
        field_info = FieldInfo(
            field=ForeignKeyNamingTestModelB._meta.get_field("incorrect_related_name"),
            name="incorrect_related_name",
            model=ForeignKeyNamingTestModelB,
        )
        check = ForeignKeyNaming(field_info)
        result = check.perform_field_check()
        self.assertIsNotNone(result)
        self.assertEqual(
            result["explanation"],
            "The field 'incorrect_related_name' should be named 'relatedmodel_id'.",
        )
